@@ -4,7 +4,7 @@ use vars qw($VERSION);
 use Text::Template::Simple::Util qw(:all);
 use Text::Template::Simple::Constants qw(:all);
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 sub _compiler { shift->[SAFE] ? COMPILER_SAFE : COMPILER }
 
@@ -126,12 +126,6 @@ sub _wrap_compile {
 
    if( $error = $@ ) {
       my $error2;
-      if ( $self->[RESUME] ) {
-         $CODE =  sub {
-                     sprintf("[%s Fatal Error] %s", $self->_class_id, $error )
-                  };
-         $error2 = $@;
-      }
       $error .= $error2 if $error2;
    }
 
@@ -170,8 +164,8 @@ Private module.
 
 =head1 DESCRIPTION
 
-This document describes version C<0.62_18> of C<Text::Template::Simple::Base::Compiler>
-released on C<26 April 2009>.
+This document describes version C<0.79_01> of C<Text::Template::Simple::Base::Compiler>
+released on C<30 April 2009>.
 
 B<WARNING>: This version of the module is part of a
 developer (beta) release of the distribution and it is
