@@ -2,7 +2,7 @@ package Text::Template::Simple::Constants;
 use strict;
 use vars qw($VERSION $OID $DID @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 # object fields
 BEGIN { $OID = -1 } # init object field id counter
@@ -32,7 +32,6 @@ use constant INSIDE_INCLUDE   => ++$OID;
 use constant NEEDS_OBJECT     => ++$OID;
 use constant CID              => ++$OID;
 use constant FILENAME         => ++$OID;
-use constant RESUME           => ++$OID;
 use constant IOLAYER          => ++$OID;
 use constant STACK            => ++$OID;
 use constant USER_THANDLER    => ++$OID;
@@ -41,6 +40,7 @@ use constant INCLUDE_PATHS    => ++$OID;
 use constant PRE_CHOMP        => ++$OID;
 use constant POST_CHOMP       => ++$OID;
 use constant VERBOSE_ERRORS   => ++$OID;
+use constant TAINT_MODE       => ++$OID;
 use constant MAXOBJFIELD      =>   $OID; # number of the last object field
 
 # token type ids
@@ -67,7 +67,6 @@ use constant DELIM_START      => 0; # field id
 use constant DELIM_END        => 1; # field id
 use constant RE_NONFILE       => qr{ [ \n \r < > \* \? ] }xmso;
 use constant RE_DUMP_ERROR    => qr{Can\'t locate object method "first" via package "B::SVOP"};
-use constant RESUME_NOSTART   => 1;                   # bool
 use constant COMPILER         => PARENT.'::Compiler'; # The compiler
 use constant COMPILER_SAFE    => COMPILER.'::Safe';   # Safe compiler
 use constant DUMMY_CLASS      => PARENT.'::Dummy';    # Dummy class
@@ -88,6 +87,11 @@ use constant COLLAPSE_LEFT    => 0x000010;
 use constant COLLAPSE_RIGHT   => 0x000020;
 use constant COLLAPSE_ALL     => 0x000040;
 
+use constant TAINT_CHECK_NORMAL  => 0x000000;
+use constant TAINT_CHECK_ALL     => 0x000002;
+use constant TAINT_CHECK_WINDOWS => 0x000004;
+use constant TAINT_CHECK_FH_READ => 0x000008;
+
 # first level directives
 use constant DIR_CAPTURE      => '=';
 use constant DIR_DYNAMIC      => '*';
@@ -129,67 +133,10 @@ use constant DIGEST_MODS => qw(
 use constant RE_PIPE_SPLIT   => qr/ \| (?:\s+)? (PARAM|FILTER) : /xms;
 use constant RE_FILTER_SPLIT => qr/ \, (?:\s+)? /xms;
 
-use constant DISK_CACHE_COMMENT => <<"TEMPLATE_CONSTANT";
-# !!!   W A R N I N G      W A R N I N G      W A R N I N G   !!!
-# This file is automatically generated by %s on %s.
-# This file is a compiled template cache.
-# Any changes you make here will be lost.
-#
-TEMPLATE_CONSTANT
-
 use constant DISK_CACHE_MARKER => q{# This file is automatically generated by }
                                .  PARENT
                                ;
 
-use constant RESUME_TEMPLATE => sub {
-   my $tmp = q~
-         .= sub {
-                  local $SIG{__DIE__};
-                  my <%RVAR%> = [];
-                  push @{ <%RVAR%> }, eval { <%TOKEN%> };
-                  return qq([<%PID%> Fatal Error] $@) if $@;
-                  return "" if(<%VOID%>);
-                  return <%RVAR%>->[0] if @{<%RVAR%>} == 1;
-                  return +(@{<%RVAR%>});
-         }->();
-   ~;
-   $tmp =~ s/\n//xmsg;
-   $tmp =~ s/\s{2,}/ /xmsg;
-   return $tmp;
-}->();
-
-use constant RESUME_MY => qr{
-      # exclude my() declarations.
-      (?:
-         (?:my|local) (?:\s+|) \(       # my($foo)
-         |
-         (?:my|local) (?:\s+|) [\$\@\%] # my $foo
-         |
-         (?:my|local)[\$\@\%]           # my$foo
-      )
-      |
-      (?:
-         (?: unless|if|while|until|for|foreach )
-         (?:\s+|)
-         \(
-      )
-}xms;
-
-use constant RESUME_CURLIES => qr{
-   \A (?:\s+|) (?:[\{\}]) (?:\s+|) \z
-}xms;
-
-use constant RESUME_ELSIF => qr{
-   \A (?:\s+|) (?:\}) (?:\s+|) (?:else|elsif)
-}xms;
-
-use constant RESUME_ELSE => qr{
-   \A (?:\s+|) \} (?:\s+|)
-   else (?:\s+|) (?:\{) (?:\s+|) \z
-}xms;
-
-use constant RESUME_LOOP => qr{ (?:next|last|continue|redo) }xms;
-
 BEGIN {
    @ISA         = qw( Exporter );
 
@@ -206,7 +153,6 @@ BEGIN {
                         PARENT
                      )],
       templates =>   [qw(
-                        DISK_CACHE_COMMENT
                         DISK_CACHE_MARKER
                      )],
       delims    =>   [qw(
@@ -241,7 +187,6 @@ BEGIN {
                         NEEDS_OBJECT
                         CID
                         FILENAME
-                        RESUME
                         IOLAYER
                         STACK
                         USER_THANDLER
@@ -250,17 +195,9 @@ BEGIN {
                         PRE_CHOMP
                         POST_CHOMP
                         VERBOSE_ERRORS
+                        TAINT_MODE
                         MAXOBJFIELD
                      )],
-      resume    =>   [qw(
-                        RESUME_NOSTART
-                        RESUME_MY
-                        RESUME_CURLIES
-                        RESUME_ELSIF
-                        RESUME_ELSE
-                        RESUME_LOOP
-                        RESUME_TEMPLATE
-                     )],
       chomp     =>   [qw(
                         CHOMP_NONE
                         COLLAPSE_NONE
@@ -306,6 +243,12 @@ BEGIN {
                         T_COMMAND
                         T_MAXID
                       )],
+      taint     =>   [qw(
+                        TAINT_CHECK_NORMAL
+                        TAINT_CHECK_ALL
+                        TAINT_CHECK_WINDOWS
+                        TAINT_CHECK_FH_READ
+                     )],
       etc       =>   [qw(
                         DIGEST_MODS
                         STAT_MTIME
@@ -340,8 +283,8 @@ TODO
 
 =head1 DESCRIPTION
 
-This document describes version C<0.62_18> of C<Text::Template::Simple::Constants>
-released on C<26 April 2009>.
+This document describes version C<0.79_01> of C<Text::Template::Simple::Constants>
+released on C<30 April 2009>.
 
 B<WARNING>: This version of the module is part of a
 developer (beta) release of the distribution and it is
@@ -355,7 +298,7 @@ Burak GE<252>rsoy, E<lt>burakE<64>cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2004-2008 Burak GE<252>rsoy. All rights reserved.
+Copyright 2004-2009 Burak GE<252>rsoy. All rights reserved.
 
 =head1 LICENSE
 
