@@ -6,17 +6,20 @@ use Text::Template::Simple::Constants qw(:all);
 use Text::Template::Simple::Util qw( DEBUG LOG ishref binary_mode fatal );
 use constant MY_IO_LAYER      => 0;
 use constant MY_INCLUDE_PATHS => 1;
+use constant MY_TAINT_MODE    => 2;
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 sub new {
    my $class = shift;
    my $layer = shift;
    my $paths = shift;
-   my $self  = [ undef, undef ];
+   my $tmode = shift;
+   my $self  = [ undef, undef, undef ];
    bless $self, $class;
    $self->[MY_IO_LAYER]      = $layer if defined $layer;
    $self->[MY_INCLUDE_PATHS] = [ @{ $paths } ] if $paths; # copy
+   $self->[MY_TAINT_MODE]    = $tmode;
    $self;
 }
 
@@ -80,12 +83,54 @@ sub slurp {
    flock $fh,    Fcntl::LOCK_SH()  if IS_FLOCK;
    seek  $fh, 0, Fcntl::SEEK_SET() if IS_FLOCK && $seek;
    $self->layer( $fh ) if ! $seek; # apply the layer only if we opened this
-   my $tmp = do { local $/; <$fh> };
+
+   if ( $self->_handle_looks_safe( $fh ) ) {
+      require IO::Handle;
+      my $rv = IO::Handle::untaint( $fh );
+      fatal('tts.io.slurp.taint') if $rv != 0;
+   }
+
+   my $tmp = do { local $/; my $rv = <$fh>; $rv };
    flock $fh, Fcntl::LOCK_UN() if IS_FLOCK;
    close $fh if ! $seek; # close only if we opened this
    return $tmp;
 }
 
+sub _handle_looks_safe {
+   # Cargo Culting: original code was taken from "The Camel"
+   my $self = shift;
+   my $fh   = shift;
+   fatal('tts.io.hls.invalid') if ! $fh || ! fileno $fh;
+
+   require File::stat;
+   my $i = File::stat::stat( $fh );
+   return if ! $i;
+
+   my $tmode = $self->[MY_TAINT_MODE];
+
+   # owner neither superuser nor "me", whose
+   # real uid is in the $< variable
+   return if $i->uid != 0 && $i->uid != $<;
+
+   # Check whether group or other can write file.
+   # Read check is disabled by default
+   # Mode always 0666 on Windows, so all tests below are disabled on Windows
+   # unless you force them to run
+   LOG( FILE_MODE => sprintf "%04o", $i->mode & 07777) if DEBUG;
+
+   my $bypass   = IS_WINDOWS && ! ( $tmode & TAINT_CHECK_WINDOWS ) ? 1 : 0;
+   my $go_write = $bypass ? 0 : $i->mode & 022;
+   my $go_read  = ! $bypass && ( $tmode & TAINT_CHECK_FH_READ )
+                ? $i->mode & 066
+                : 0;
+
+   LOG( TAINT => "tmode:$tmode; bypass:$bypass; "
+                ."go_write:$go_write; go_read:$go_read") if DEBUG;
+
+   return if $go_write || $go_read;
+   return 1;
+}
+
 sub is_file {
    # safer than a simple "-e"
    my $self = shift;
@@ -145,8 +190,8 @@ TODO
 
 =head1 DESCRIPTION
 
-This document describes version C<0.62_18> of C<Text::Template::Simple::IO>
-released on C<26 April 2009>.
+This document describes version C<0.79_01> of C<Text::Template::Simple::IO>
+released on C<30 April 2009>.
 
 B<WARNING>: This version of the module is part of a
 developer (beta) release of the distribution and it is
@@ -184,7 +229,8 @@ Test if C<THING> is a file.
 
 =head2 file_exists THING
 
-Test if C<THING> is a file. This method also searches all the C<include paths>.
+Test if C<THING> is a file. This method also searches all the C<include paths>
+and returns the full path to the file if it exists.
 
 =head1 AUTHOR
 
