@@ -4,7 +4,7 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 use Text::Template::Simple::Constants qw( :info DIGEST_MODS );
 use Carp qw( croak );
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 BEGIN {
    if ( IS_WINDOWS ) {
@@ -74,6 +74,8 @@ my $lang = {
       'tts.io.validate.file'                     => 'validate(file) is not yet implemented',
       'tts.io.layer.fh'                          => 'Filehandle is absent',
       'tts.io.slurp.open'                        => "Error opening '%s' for reading: %s",
+      'tts.io.slurp.taint'                       => "Can't untaint FH",
+      'tts.io.hls.invalid'                       => 'FH is either absent or invalid',
       'tts.caller.stack.hash'                    => 'Parameters to stack() must be a HASH',
       'tts.caller.stack.type'                    => 'Unknown caller stack type: %s',
       'tts.caller._text_table.module'            => "Caller stack type 'text_table' requires Text::Table: %s",
@@ -225,8 +227,8 @@ TODO
 
 =head1 DESCRIPTION
 
-This document describes version C<0.62_18> of C<Text::Template::Simple::Util>
-released on C<26 April 2009>.
+This document describes version C<0.79_01> of C<Text::Template::Simple::Util>
+released on C<30 April 2009>.
 
 B<WARNING>: This version of the module is part of a
 developer (beta) release of the distribution and it is
