@@ -33,7 +33,7 @@ package Text::Template::Simple::Constants;
 use strict;
 use vars qw($VERSION $OID $DID @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 # object fields
 BEGIN { $OID = -1 } # init object field id counter
@@ -63,7 +63,6 @@ use constant INSIDE_INCLUDE   => ++$OID;
 use constant NEEDS_OBJECT     => ++$OID;
 use constant CID              => ++$OID;
 use constant FILENAME         => ++$OID;
-use constant RESUME           => ++$OID;
 use constant IOLAYER          => ++$OID;
 use constant STACK            => ++$OID;
 use constant USER_THANDLER    => ++$OID;
@@ -72,6 +71,7 @@ use constant INCLUDE_PATHS    => ++$OID;
 use constant PRE_CHOMP        => ++$OID;
 use constant POST_CHOMP       => ++$OID;
 use constant VERBOSE_ERRORS   => ++$OID;
+use constant TAINT_MODE       => ++$OID;
 use constant MAXOBJFIELD      =>   $OID; # number of the last object field
 
 # token type ids
@@ -98,7 +98,6 @@ use constant DELIM_START      => 0; # field id
 use constant DELIM_END        => 1; # field id
 use constant RE_NONFILE       => qr{ [ \n \r < > \* \? ] }xmso;
 use constant RE_DUMP_ERROR    => qr{Can\'t locate object method "first" via package "B::SVOP"};
-use constant RESUME_NOSTART   => 1;                   # bool
 use constant COMPILER         => PARENT.'::Compiler'; # The compiler
 use constant COMPILER_SAFE    => COMPILER.'::Safe';   # Safe compiler
 use constant DUMMY_CLASS      => PARENT.'::Dummy';    # Dummy class
@@ -119,6 +118,11 @@ use constant COLLAPSE_LEFT    => 0x000010;
 use constant COLLAPSE_RIGHT   => 0x000020;
 use constant COLLAPSE_ALL     => 0x000040;
 
+use constant TAINT_CHECK_NORMAL  => 0x000000;
+use constant TAINT_CHECK_ALL     => 0x000002;
+use constant TAINT_CHECK_WINDOWS => 0x000004;
+use constant TAINT_CHECK_FH_READ => 0x000008;
+
 # first level directives
 use constant DIR_CAPTURE      => '=';
 use constant DIR_DYNAMIC      => '*';
@@ -160,67 +164,10 @@ use constant DIGEST_MODS => qw(
 use constant RE_PIPE_SPLIT   => qr/ \| (?:\s+)? (PARAM|FILTER) : /xms;
 use constant RE_FILTER_SPLIT => qr/ \, (?:\s+)? /xms;
 
-use constant DISK_CACHE_COMMENT => <<"TEMPLATE_CONSTANT";
-# !!!   W A R N I N G      W A R N I N G      W A R N I N G   !!!
-# This file is automatically generated by %s on %s.
-# This file is a compiled template cache.
-# Any changes you make here will be lost.
-#
-TEMPLATE_CONSTANT
-
 use constant DISK_CACHE_MARKER => q{# This file is automatically generated by }
                                .  PARENT
                                ;
 
-use constant RESUME_TEMPLATE => sub {
-   my $tmp = q~
-         .= sub {
-                  local $SIG{__DIE__};
-                  my <%RVAR%> = [];
-                  push @{ <%RVAR%> }, eval { <%TOKEN%> };
-                  return qq([<%PID%> Fatal Error] $@) if $@;
-                  return "" if(<%VOID%>);
-                  return <%RVAR%>->[0] if @{<%RVAR%>} == 1;
-                  return +(@{<%RVAR%>});
-         }->();
-   ~;
-   $tmp =~ s/\n//xmsg;
-   $tmp =~ s/\s{2,}/ /xmsg;
-   return $tmp;
-}->();
-
-use constant RESUME_MY => qr{
-      # exclude my() declarations.
-      (?:
-         (?:my|local) (?:\s+|) \(       # my($foo)
-         |
-         (?:my|local) (?:\s+|) [\$\@\%] # my $foo
-         |
-         (?:my|local)[\$\@\%]           # my$foo
-      )
-      |
-      (?:
-         (?: unless|if|while|until|for|foreach )
-         (?:\s+|)
-         \(
-      )
-}xms;
-
-use constant RESUME_CURLIES => qr{
-   \A (?:\s+|) (?:[\{\}]) (?:\s+|) \z
-}xms;
-
-use constant RESUME_ELSIF => qr{
-   \A (?:\s+|) (?:\}) (?:\s+|) (?:else|elsif)
-}xms;
-
-use constant RESUME_ELSE => qr{
-   \A (?:\s+|) \} (?:\s+|)
-   else (?:\s+|) (?:\{) (?:\s+|) \z
-}xms;
-
-use constant RESUME_LOOP => qr{ (?:next|last|continue|redo) }xms;
-
 BEGIN {
    @ISA         = qw( Exporter );
 
@@ -237,7 +184,6 @@ BEGIN {
                         PARENT
                      )],
       templates =>   [qw(
-                        DISK_CACHE_COMMENT
                         DISK_CACHE_MARKER
                      )],
       delims    =>   [qw(
@@ -272,7 +218,6 @@ BEGIN {
                         NEEDS_OBJECT
                         CID
                         FILENAME
-                        RESUME
                         IOLAYER
                         STACK
                         USER_THANDLER
@@ -281,17 +226,9 @@ BEGIN {
                         PRE_CHOMP
                         POST_CHOMP
                         VERBOSE_ERRORS
+                        TAINT_MODE
                         MAXOBJFIELD
                      )],
-      resume    =>   [qw(
-                        RESUME_NOSTART
-                        RESUME_MY
-                        RESUME_CURLIES
-                        RESUME_ELSIF
-                        RESUME_ELSE
-                        RESUME_LOOP
-                        RESUME_TEMPLATE
-                     )],
       chomp     =>   [qw(
                         CHOMP_NONE
                         COLLAPSE_NONE
@@ -337,6 +274,12 @@ BEGIN {
                         T_COMMAND
                         T_MAXID
                       )],
+      taint     =>   [qw(
+                        TAINT_CHECK_NORMAL
+                        TAINT_CHECK_ALL
+                        TAINT_CHECK_WINDOWS
+                        TAINT_CHECK_FH_READ
+                     )],
       etc       =>   [qw(
                         DIGEST_MODS
                         STAT_MTIME
@@ -363,7 +306,7 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 use Text::Template::Simple::Constants qw( :info DIGEST_MODS );
 use Carp qw( croak );
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 BEGIN {
    if ( IS_WINDOWS ) {
@@ -433,6 +376,8 @@ my $lang = {
       'tts.io.validate.file'                     => 'validate(file) is not yet implemented',
       'tts.io.layer.fh'                          => 'Filehandle is absent',
       'tts.io.slurp.open'                        => "Error opening '%s' for reading: %s",
+      'tts.io.slurp.taint'                       => "Can't untaint FH",
+      'tts.io.hls.invalid'                       => 'FH is either absent or invalid',
       'tts.caller.stack.hash'                    => 'Parameters to stack() must be a HASH',
       'tts.caller.stack.type'                    => 'Unknown caller stack type: %s',
       'tts.caller._text_table.module'            => "Caller stack type 'text_table' requires Text::Table: %s",
@@ -576,7 +521,7 @@ use strict;
 use vars qw($VERSION);
 use Text::Template::Simple::Dummy;
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 sub _compile { shift; return __PACKAGE__->_object->reval(shift) }
 
@@ -609,7 +554,7 @@ use overload q{""} => 'get';
 use Text::Template::Simple::Constants qw( MAX_FL );
 use Text::Template::Simple::Util      qw( DIGEST fatal );
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 my $RE_INVALID = qr{[^A-Za-z_0-9]};
 
@@ -648,111 +593,12 @@ package Text::Template::Simple::Base::Parser;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
-use Text::Template::Simple::Util qw(:all);
+use Text::Template::Simple::Util      qw(:all);
 use Text::Template::Simple::Constants qw(:all);
 
-# internal code templates
-my %INTERNAL = (
-   # we need string eval in this template to catch syntax errors
-   sub_include => q~
-      <%OBJECT%>->_compile(
-         do {
-            local $@;
-            my $file = eval '<%INCLUDE%>';
-            my $rv;
-            if ( my $e = $@ ) {
-               chomp $e;
-               $file ||= '<%INCLUDE%>';
-               my $m = "The parameter ($file) is not a file. "
-                     . "Error from sub-include ($file): $e";
-               $rv = [ ERROR => '<%ERROR_TITLE%> ' . $m ]
-            }
-            else {
-               $rv = $file;
-            }
-            $rv;
-         },
-         <%PARAMS%>,
-         {
-            _sub_inc => '<%TYPE%>',
-            _filter  => '<%FILTER%>',
-         }
-      )
-   ~,
-   no_monolith => q*
-      <%OBJECT%>->compile(
-         q~<%FILE%>~,
-         undef,
-         {
-            chkmt    => 1,
-            _sub_inc => q~<%TYPE%>~,
-         }
-      );
-   *,
-
-   # see _parse()
-   map_keys_check => q(
-      <%BUF%> .= exists <%HASH%>->{"<%KEY%>"}
-               ? (
-                  defined <%HASH%>->{"<%KEY%>"}
-                  ? <%HASH%>->{"<%KEY%>"}
-                  : "[ERROR] Key not defined: <%KEY%>"
-                  )
-               : "[ERROR] Invalid key: <%KEY%>"
-               ;
-   ),
-
-   map_keys_init => q(
-      <%BUF%> .= <%HASH%>->{"<%KEY%>"} || '';
-   ),
-   map_keys_default => q(
-      <%BUF%> .= <%HASH%>->{"<%KEY%>"};
-   ),
-
-   add_sigwarn => q(
-      my <%BUF%>;
-      local $SIG{__WARN__} = sub {
-         push @{ <%BUF%> }, $_[0];
-      };
-   ),
-   dump_sigwarn => q(
-      join("\n",
-            map {
-               s{ \A \s+    }{}xms;
-               s{    \s+ \z }{}xms;
-               "[warning] $_\n"
-            } @{ <%BUF%> }
-         );
-   ),
-
-   compile_error => <<'TEMPLATE_CONSTANT',
-Error compiling code fragment (cache id: <%CID%>):
-
-<%ERROR%>
--------------------------------
-PARSED CODE (VERBATIM):
--------------------------------
-
-<%PARSED%>
-
--------------------------------
-PARSED CODE    (tidied):
--------------------------------
-
-<%TIDIED%>
-TEMPLATE_CONSTANT
-   fragment => <<'TEMPLATE_CONSTANT',
-
-# BEGIN TIDIED FRAGMENT
-
-<%FRAGMENT%>
-
-# END TIDIED FRAGMENT
-TEMPLATE_CONSTANT
-
-);
+my %INTERNAL = __PACKAGE__->_set_internal_templates;
 
 sub _internal {
    my $self = shift;
@@ -769,9 +615,7 @@ sub _parse {
    my $as_is    = shift; # i.e.: do not parse -> static include
    #$self->[NEEDS_OBJECT] = 0; # reset
 
-   my $resume   = $self->[RESUME] || '';
-   my $ds       = $self->[DELIMITERS][DELIM_START];
-   my $de       = $self->[DELIMITERS][DELIM_END  ];
+   my($ds, $de) = @{ $self->[DELIMITERS] };
    my $faker    = $self->[INSIDE_INCLUDE] ? $self->_output_buffer_var
                                           : $self->[FAKER]
                                           ;
@@ -786,7 +630,7 @@ sub _parse {
 
    LOG( RAW => $raw ) if ( DEBUG() > 3 );
 
-   my $uthandler = $self->[USER_THANDLER];
+   my $uth = $self->[USER_THANDLER];
 
    my $h = {
       raw     => sub { ";$faker .= q~$_[0]~;" },
@@ -812,18 +656,15 @@ sub _parse {
       }
 
       elsif ( T_CODE == $id ) {
-         $code .= $h->{code}->($resume ? $self->_resume($str, 0, 1) : $str);
+         $code .= $h->{code}->($str);
       }
 
       elsif ( T_CAPTURE == $id ) {
-         $code .= $faker;
-         $code .= $resume ? $self->_resume($str, RESUME_NOSTART)
-                :           " .= sub { $str }->();";
+         $code .= $h->{capture}->( $str );
       }
 
       elsif ( T_DYNAMIC == $id || T_STATIC == $id ) {
-         $self->[NEEDS_OBJECT]++;
-         $code .= $h->{capture}->( $self->_include($id, $str) );
+         $code .= $h->{capture}->( $self->_needs_object->_include($id, $str) );
       }
 
       elsif ( T_MAPKEY == $id ) {
@@ -831,33 +672,15 @@ sub _parse {
       }
 
       elsif ( T_COMMAND == $id ) {
-         my($head, $raw_block) = split /;/, $str, 2;
-         my @buf = split RE_PIPE_SPLIT, '|' . trim($head);
-         shift(@buf);
-         my %com = map { trim $_ } @buf;
-
-         if ( $com{FILTER} ) {
-            # embed into the template & NEEDS_OBJECT++ ???
-            local $self->[FILENAME] = '<ANON BLOCK>';
-            $self->_call_filters(
-               \$raw_block,
-               split RE_FILTER_SPLIT, $com{FILTER}
-            );
-         }
-
-         $code .= $h->{raw}->($raw_block);
+         $code .= $h->{raw}->( $self->_parse_command( $str ) );
       }
 
       else {
-         if ( $uthandler ) {
-            LOG( USER_THANDLER => "$id") if DEBUG();
-            $code .= $uthandler->( $self, $id ,$str, $h );
-         }
-         else {
-            LOG( UNKNOWN_TOKEN => "Adding unknown token as RAW: $id($str)")
-               if DEBUG();
-            $code .= $h->{raw}->($str);
-         }
+         LOG(
+            $uth  ? (USER_THANDLER => "$id")
+                  : (UNKNOWN_TOKEN => "Adding unknown token as RAW: $id($str)")
+         ) if DEBUG;
+         $code .= $uth ? $uth->( $self, $id ,$str, $h ) : $h->{raw}->( $str );
       }
 
    }
@@ -874,6 +697,23 @@ sub _parse {
    return $self->_wrapper( $code, $cache_id, $faker, $map_keys, $h );
 }
 
+sub _parse_command {
+   my $self = shift;
+   my $str  = shift;
+   my($head, $raw_block) = split /;/, $str, 2;
+   my @buf  = split RE_PIPE_SPLIT, '|' . trim($head);
+   shift(@buf);
+   my %com  = map { trim $_ } @buf;
+
+   if ( $com{FILTER} ) {
+      # embed into the template & NEEDS_OBJECT++ ???
+      local $self->[FILENAME] = '<ANON BLOCK>';
+      $self->_call_filters( \$raw_block, split RE_FILTER_SPLIT, $com{FILTER} );
+   }
+
+   return $raw_block;
+}
+
 sub _chomp {
    # remove the unnecessary white space
    my $self = shift;
@@ -1024,41 +864,112 @@ sub _add_stack {
    return "$channel stack( { type => '$type', name => '$cs_name' } );";
 }
 
-# TODO: unstable. consider removing this thing (also the constants)
-sub _resume {
-   my $self    = shift;
-   my $token   = shift           || return;
-   my $nostart = shift           || 0;
-   my $is_code = shift           || 0;
-   my $resume  = $self->[RESUME] || '';
-   my $start   = $nostart ? '' : $self->[FAKER];
-   my $void    = $nostart ? 0  : 1; # not a self-printing block
-
-   if ( $token && $resume && $token !~ RESUME_MY ) {
-      if (
-            $token !~ RESUME_CURLIES &&
-            $token !~ RESUME_ELSIF   &&
-            $token !~ RESUME_ELSE    &&
-            $token !~ RESUME_LOOP
-      ) {
-         LOG( RESUME_OK => $token ) if DEBUG() > 2;
-         my $rvar        = $self->_output_buffer_var('array');
-         my $resume_code = RESUME_TEMPLATE;
-         foreach my $replace (
-            [ RVAR  => $rvar             ],
-            [ TOKEN => $token            ],
-            [ PID   => $self->_class_id  ],
-            [ VOID  => $void             ],
-         ) {
-            $resume_code =~ s{ <% $replace->[0] %> }{$replace->[1]}xmsg;
+sub _set_internal_templates {
+   # we need string eval in this template to catch syntax errors
+   sub_include => q~
+      <%OBJECT%>->_compile(
+         do {
+            local $@;
+            my $file = eval '<%INCLUDE%>';
+            my $rv;
+            if ( my $e = $@ ) {
+               chomp $e;
+               $file ||= '<%INCLUDE%>';
+               my $m = "The parameter ($file) is not a file. "
+                     . "Error from sub-include ($file): $e";
+               $rv = [ ERROR => '<%ERROR_TITLE%> ' . $m ]
+            }
+            else {
+               $rv = $file;
+            }
+            $rv;
+         },
+         <%PARAMS%>,
+         {
+            _sub_inc => '<%TYPE%>',
+            _filter  => '<%FILTER%>',
          }
-         return $start . $resume_code;
-      }
-   }
+      )
+   ~,
+   no_monolith => q*
+      <%OBJECT%>->compile(
+         q~<%FILE%>~,
+         undef,
+         {
+            chkmt    => 1,
+            _sub_inc => q~<%TYPE%>~,
+         }
+      );
+   *,
 
-   LOG( RESUME_NOT => $token ) if DEBUG() > 2;
+   # see _parse()
+   map_keys_check => q(
+      <%BUF%> .= exists <%HASH%>->{"<%KEY%>"}
+               ? (
+                  defined <%HASH%>->{"<%KEY%>"}
+                  ? <%HASH%>->{"<%KEY%>"}
+                  : "[ERROR] Key not defined: <%KEY%>"
+                  )
+               : "[ERROR] Invalid key: <%KEY%>"
+               ;
+   ),
 
-   return $is_code ? $token : "$start .= $token;"
+   map_keys_init => q(
+      <%BUF%> .= <%HASH%>->{"<%KEY%>"} || '';
+   ),
+   map_keys_default => q(
+      <%BUF%> .= <%HASH%>->{"<%KEY%>"};
+   ),
+
+   add_sigwarn => q(
+      my <%BUF%>;
+      local $SIG{__WARN__} = sub {
+         push @{ <%BUF%> }, $_[0];
+      };
+   ),
+   dump_sigwarn => q(
+      join("\n",
+            map {
+               s{ \A \s+    }{}xms;
+               s{    \s+ \z }{}xms;
+               "[warning] $_\n"
+            } @{ <%BUF%> }
+         );
+   ),
+
+   compile_error => <<'TEMPLATE_CONSTANT',
+Error compiling code fragment (cache id: <%CID%>):
+
+<%ERROR%>
+-------------------------------
+PARSED CODE (VERBATIM):
+-------------------------------
+
+<%PARSED%>
+
+-------------------------------
+PARSED CODE    (tidied):
+-------------------------------
+
+<%TIDIED%>
+TEMPLATE_CONSTANT
+
+   fragment => <<'TEMPLATE_CONSTANT',
+
+# BEGIN TIDIED FRAGMENT
+
+<%FRAGMENT%>
+
+# END TIDIED FRAGMENT
+TEMPLATE_CONSTANT
+
+   disk_cache_comment => <<"TEMPLATE_CONSTANT",
+# !!!   W A R N I N G      W A R N I N G      W A R N I N G   !!!
+# This file was automatically generated by <%NAME%> on <%DATE%>.
+# This file is a compiled template cache.
+# Any changes you make here will be lost.
+#
+TEMPLATE_CONSTANT
 }
 
 package Text::Template::Simple::Base::Include;
@@ -1067,7 +978,7 @@ use vars qw($VERSION);
 use Text::Template::Simple::Util qw(:all);
 use Text::Template::Simple::Constants qw(:all);
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 sub _include_no_monolith {
    # no monolith eh?
@@ -1230,7 +1141,7 @@ use vars qw($VERSION);
 use Text::Template::Simple::Util qw(:all);
 use Text::Template::Simple::Constants qw(:all);
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 sub _examine {
    my $self   = shift;
@@ -1297,7 +1208,7 @@ use vars qw($VERSION);
 use Text::Template::Simple::Util qw(:all);
 use Text::Template::Simple::Constants qw(:all);
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 sub _compiler { shift->[SAFE] ? COMPILER_SAFE : COMPILER }
 
@@ -1419,12 +1330,6 @@ sub _wrap_compile {
 
    if( $error = $@ ) {
       my $error2;
-      if ( $self->[RESUME] ) {
-         $CODE =  sub {
-                     sprintf("[%s Fatal Error] %s", $self->_class_id, $error )
-                  };
-         $error2 = $@;
-      }
       $error .= $error2 if $error2;
    }
 
@@ -1453,7 +1358,7 @@ package Text::Template::Simple::Tokenizer;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 use constant CMD_CHAR             =>  0;
 use constant CMD_ID               =>  1;
@@ -1772,17 +1677,20 @@ use Text::Template::Simple::Constants qw(:all);
 use Text::Template::Simple::Util qw( DEBUG LOG ishref binary_mode fatal );
 use constant MY_IO_LAYER      => 0;
 use constant MY_INCLUDE_PATHS => 1;
+use constant MY_TAINT_MODE    => 2;
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 sub new {
    my $class = shift;
    my $layer = shift;
    my $paths = shift;
-   my $self  = [ undef, undef ];
+   my $tmode = shift;
+   my $self  = [ undef, undef, undef ];
    bless $self, $class;
    $self->[MY_IO_LAYER]      = $layer if defined $layer;
    $self->[MY_INCLUDE_PATHS] = [ @{ $paths } ] if $paths; # copy
+   $self->[MY_TAINT_MODE]    = $tmode;
    $self;
 }
 
@@ -1846,12 +1754,54 @@ sub slurp {
    flock $fh,    Fcntl::LOCK_SH()  if IS_FLOCK;
    seek  $fh, 0, Fcntl::SEEK_SET() if IS_FLOCK && $seek;
    $self->layer( $fh ) if ! $seek; # apply the layer only if we opened this
-   my $tmp = do { local $/; <$fh> };
+
+   if ( $self->_handle_looks_safe( $fh ) ) {
+      require IO::Handle;
+      my $rv = IO::Handle::untaint( $fh );
+      fatal('tts.io.slurp.taint') if $rv != 0;
+   }
+
+   my $tmp = do { local $/; my $rv = <$fh>; $rv };
    flock $fh, Fcntl::LOCK_UN() if IS_FLOCK;
    close $fh if ! $seek; # close only if we opened this
    return $tmp;
 }
 
+sub _handle_looks_safe {
+   # Cargo Culting: original code was taken from "The Camel"
+   my $self = shift;
+   my $fh   = shift;
+   fatal('tts.io.hls.invalid') if ! $fh || ! fileno $fh;
+
+   require File::stat;
+   my $i = File::stat::stat( $fh );
+   return if ! $i;
+
+   my $tmode = $self->[MY_TAINT_MODE];
+
+   # owner neither superuser nor "me", whose
+   # real uid is in the $< variable
+   return if $i->uid != 0 && $i->uid != $<;
+
+   # Check whether group or other can write file.
+   # Read check is disabled by default
+   # Mode always 0666 on Windows, so all tests below are disabled on Windows
+   # unless you force them to run
+   LOG( FILE_MODE => sprintf "%04o", $i->mode & 07777) if DEBUG;
+
+   my $bypass   = IS_WINDOWS && ! ( $tmode & TAINT_CHECK_WINDOWS ) ? 1 : 0;
+   my $go_write = $bypass ? 0 : $i->mode & 022;
+   my $go_read  = ! $bypass && ( $tmode & TAINT_CHECK_FH_READ )
+                ? $i->mode & 066
+                : 0;
+
+   LOG( TAINT => "tmode:$tmode; bypass:$bypass; "
+                ."go_write:$go_write; go_read:$go_read") if DEBUG;
+
+   return if $go_write || $go_read;
+   return 1;
+}
+
 sub is_file {
    # safer than a simple "-e"
    my $self = shift;
@@ -1909,7 +1859,7 @@ use vars qw($VERSION);
 use Text::Template::Simple::Caller;
 use Text::Template::Simple::Util qw();
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 sub stack { # just a wrapper
    my $opt = shift || {};
@@ -1925,7 +1875,7 @@ use strict;
 use vars qw($VERSION);
 use Text::Template::Simple::Dummy;
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 sub _compile { shift; return eval shift }
 
@@ -1944,7 +1894,7 @@ use constant HINTS      => 8;
 use constant BITMASK    => 9;
 use Text::Template::Simple::Util qw( ishref fatal );
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 sub stack {
    my $self    = shift;
@@ -2117,7 +2067,7 @@ use Text::Template::Simple::Constants qw(:all);
 use Text::Template::Simple::Util qw( DEBUG LOG ishref fatal );
 use Carp qw( croak );
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 my $CACHE = {}; # in-memory template cache
 
@@ -2159,8 +2109,8 @@ sub reset {
       my $file;
 
       while ( defined( $file = readdir CDIRH ) ) {
-         next if $file !~ m{ $ext \z}xmsi;
-         $file = File::Spec->catfile( $parent->[CACHE_DIR], $file );
+         next if $file !~ m{ ( .* $ext) \z}xmsi;
+         $file = File::Spec->catfile( $parent->[CACHE_DIR], $1 );
          LOG( UNLINK => $file ) if DEBUG();
          unlink $file;
       }
@@ -2377,6 +2327,12 @@ sub hit {
             %meta = $self->_get_meta( $1 );
             fatal('tts.cache.hit.meta', $@) if $@;
          }
+         if ( ! $meta{VERSION} || $meta{VERSION} + 0 < PARENT->VERSION ) {
+            my $id = $parent->[FILENAME] || $cache_id;
+            warn "(This messeage will only appear once) $id was compiled with"
+                ." an old version of " . PARENT . ". Resetting cache.";
+            return;
+         }
          if ( my $mtime = $meta{CHKMT} ) {
             if ( $mtime != $chkmt ) {
                LOG( MTIME_DIFF => "\tOLD: $mtime\n\t\tNEW: $chkmt")
@@ -2430,6 +2386,7 @@ sub populate {
             CHKMT        => $chkmt,
             NEEDS_OBJECT => $parent->[NEEDS_OBJECT],
             FAKER_SELF   => $parent->[FAKER_SELF],
+            VERSION      => PARENT->VERSION,
          );
 
          my $cache = File::Spec->catfile( $cdir, $cache_id . CACHE_EXT);
@@ -2437,10 +2394,14 @@ sub populate {
          $fh->open($cache, '>') or fatal('tts.cache.populate.write', $cache, $!);
          flock $fh, Fcntl::LOCK_EX() if IS_FLOCK;
          $parent->io->layer($fh);
-         print $fh '#META:' . $self->_set_meta(\%meta) . "\n",
-                   sprintf( DISK_CACHE_COMMENT,
-                            PARENT->_class_id, scalar localtime time),
-                   $parsed; 
+         my $warn =  $parent->_mini_compiler(
+                        $parent->_internal('disk_cache_comment'),
+                        {
+                           NAME => PARENT->_class_id,
+                           DATE => scalar localtime time,
+                        }
+                     );
+         print $fh '#META:' . $self->_set_meta(\%meta) . "\n", $warn, $parsed; 
          flock $fh, Fcntl::LOCK_UN() if IS_FLOCK;
          close $fh;
 
@@ -2506,7 +2467,7 @@ package Text::Template::Simple;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '0.62_18';
+$VERSION = '0.79_01';
 
 use File::Spec;
 use Text::Template::Simple::Constants qw(:all);
@@ -2552,8 +2513,7 @@ my %DEFAULT = ( # default object attributes
    verbose_errors   =>  0,    # bool
    pre_chomp        => CHOMP_NONE,
    post_chomp       => CHOMP_NONE,
-   # TODO: Consider removing this
-   resume           =>  0,    # resume on error?
+   taint_mode       => TAINT_CHECK_NORMAL,
 );
 
 my @EXPORT_OK = qw( tts );
@@ -2659,7 +2619,8 @@ sub _init {
 
    $self->[IO_OBJECT] = $self->connector('IO')->new(
                            $self->[IOLAYER],
-                           $self->[INCLUDE_PATHS]
+                           $self->[INCLUDE_PATHS],
+                           $self->[TAINT_MODE],
                         );
 
    if ( $self->[CACHE_DIR] ) {
@@ -2720,6 +2681,12 @@ sub _tidy {
    return $buf;
 }
 
+sub _needs_object {
+   my $self = shift;
+   $self->[NEEDS_OBJECT]++;
+   return $self;
+}
+
 sub DESTROY {
    my $self = shift || return;
    LOG( DESTROY => ref $self ) if DEBUG();
@@ -2762,18 +2729,19 @@ generated with an automatic build tool. If you experience problems
 with this version, please install and use the supported standard
 version. This version is B<NOT SUPPORTED>.
 
-This document describes version C<0.62_18> of C<Text::Template::Simple>
-released on C<26 April 2009>.
+This document describes version C<0.79_01> of C<Text::Template::Simple>
+released on C<30 April 2009>.
 
 B<WARNING>: This version of the module is part of a
 developer (beta) release of the distribution and it is
 not suitable for production use.
 
 This is a simple template module. There is no extra template/mini 
-language. Instead, it uses Perl as a template language. Templates
-can be cached on disk or inside the memory via internal cache 
+language. Instead, it uses Perl as the template language. Templates
+can be cached on disk or inside the memory via the internal cache 
 manager. It is also possible to use static/dynamic includes,
-pass parameters to includes and allpt filters on them.
+pass parameters to includes and apply filters on them.
+Also see L<Text::Template::Simple::API> for the full API definiton.
 
 =head1 SYNTAX
 
@@ -2783,31 +2751,31 @@ Template syntax is very simple. There are few kinds of delimiters:
 
 =item *
 
-Code Blocks: C<< <% %> >>
+C<< <% %>  >> Code Blocks
 
 =item *
 
-Self-printing Blocks: C<< <%= %> >>
+C<< <%= %> >> Self-printing Blocks
 
 =item *
 
-Escaped Delimiters: C<< <%! %> >>
+C<< <%! %> >> Escaped Delimiters
 
 =item *
 
-Static Include Directives: C<< <%+ %> >>
+C<< <%+ %> >> Static Include Directives
 
 =item *
 
-Dynamic include directives C<< <%* %> >>
+C<< <%* %> >> Dynamic include directives
 
 =item *
 
-Comment Directives: C<< <%# %> >>
+C<< <%# %> >> Comment Directives
 
 =item *
 
-Blocks with commands: C<< <%| %> >>.
+C<< <%| %> >> Blocks with commands
 
 =back
 
@@ -2866,7 +2834,7 @@ parsing for the included files, use the dynamic includes:
    <%* my_other.html %>
    <%* my_other.txt  %>
 
-Interpolation is also supported with both kind of includes, so the following
+Interpolation is also supported with both kinds of includes, so the following
 is valid code:
 
    <%+ "/path/to/" . $txt    %>
@@ -3152,7 +3120,7 @@ Burak GE<252>rsoy, E<lt>burakE<64>cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2004-2008 Burak GE<252>rsoy. All rights reserved.
+Copyright 2004-2009 Burak GE<252>rsoy. All rights reserved.
 
 =head1 LICENSE
 
